/*
    DebugQueueM module -  that buffers sending to a device eg UART.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**P1-D9C9U9T9 P2-D1C0U0T0 Dbg Output buffering
 * 
  Simple Console component that buffers a device eg Uart interface. Buffer is 512 chars.
  <p>This configuration maps onto the uart that is normally used to connect onto 
  a pc.</p>

   [P1] is buffering the debug output to the UART
   [P2] is adding the ability to send it out as a debug message to a
   gateway.
   
  <p>Please note that this is a simple component for fast debugging.</p>
  <p> Testing: This module has been tested for a number of conditions including <b>
  large buffers - throws away the input
  repeated large data requests into the queue.
  Reducing the uartQueuBuffer to small sizes and verifying the response
  
 */

#include "main.h"
#include "n_ttyQueue.h"

static void outputNextChar();

#ifdef DEBUG_QUELG
#define SIMPLEQUE_UARTQUEUEBUFFER_SIZE 3000
#elif defined(DEBUG_QUE5000)
#define SIMPLEQUE_UARTQUEUEBUFFER_SIZE 5000
#else 
#define SIMPLEQUE_UARTQUEUEBUFFER_SIZE 1000
#endif 
  /** The buffer used to store for transmission */
static  char * pQueueHead; // were the next character for the UART is
static   char * pQueueTail; // were the next space is for a character
 
static  uint8_t isOutputting;
static  uint8_t isHibernating; //bool FALSE is active - atomic as 1bit unless r/w
static  char uartQueueBuffer[SIMPLEQUE_UARTQUEUEBUFFER_SIZE];
static  int curCount; //number of characters in buffer
static  int16_t totCount; //Stts - characters sent on the uart
#define BUFFER_END_ADDR  uartQueueBuffer+SIMPLEQUE_UARTQUEUEBUFFER_SIZE

/*dc***************************************************/
  result_t ttyQueue_init() {
  /* Init */

    disable_irq(); {
      pQueueHead   = uartQueueBuffer;
      pQueueTail   = uartQueueBuffer;
      isOutputting = FALSE;
      isHibernating = FALSE;
      curCount = totCount        = 0;
      
    } enable_irq();
    return SUCCESS;
  }
 /*df***************************************************/
inline uint16_t ttyQueCntl_available() { return (SIMPLEQUE_UARTQUEUEBUFFER_SIZE-curCount);}
inline uint16_t dbgQue_size() {return curCount;}
void ttyQueHibernateSs_start() { }
void ttyQueHibernateSs_end() {isHibernating = FALSE;}

/*df***************************************************/
inline static void putBufC(const uint8_t uchr8) {
/* put char into uartQueueBuffer if available space - else throw it away
   char is pushed into pQueueTail */

   isOutputting = TRUE;
   //DebugSerial_put(uchr8);
   ttyHalPutc(uchr8);

   if (--curCount < 0) curCount = 0;
   totCount++;
}
/*df***************************************************/
inline static void outputHeadChar() {
    putBufC(*pQueueHead);
    /* Adjust pQueueHead */
    ++pQueueHead;
    //++totCount;

    if (pQueueHead >= BUFFER_END_ADDR) {
      pQueueHead = uartQueueBuffer;
    }
}
/*df***************************************************/
 uint8_t ttyQueue_putc(const uint8_t uchr8) {
/* put char into uartQueueBuffer if available space - else throw it away
  char is pushed into pQueueTail  */
  uint8_t retResult=1;

  disable_irq(); //  {

    if (curCount >= (SIMPLEQUE_UARTQUEUEBUFFER_SIZE-1)) {
      retResult = 0;
    } else {  
      *pQueueTail = uchr8;
      pQueueTail++;
      curCount++;
      if (pQueueTail >=  BUFFER_END_ADDR) {
        pQueueTail = uartQueueBuffer;
      }
      if (!isOutputting) {
    	  /*TODO make dependent on 1-2mS having passed since last char
    	   * ttyLastUpdate_tick = HAL_GetTick();
    	   * then from SysTick_Handler() poll for period on update
    	  */
    	  outputHeadChar();
      }
    }
  enable_irq();//}
  return retResult;
}

/*df***************************************************/
inline static void outputNextChar() {//Simple
  /* Handle emptying the uartQueueBuffer - the character at pQueueHead has now been outputted 
     and we need to output the next, if needed. */
	disable_irq();
	{
    /* Check for more bytes */
    if (pQueueHead != pQueueTail) {
    	outputHeadChar();
    } else {
      isOutputting = FALSE;
      if (TRUE== isHibernating) {
        //TODO to gointo Hibernating
      }
    }
  } enable_irq();
}
/*de***************************************************/
/*async event*/ void ttyQueue_putDone() {
  outputNextChar();
}




